
#include "tap.h"

#include "ibox.gd"
#include "vol.gd"

#define ID_MYOSD 0x80000109

TAP_ID( ID_MYOSD );
TAP_PROGRAM_NAME("My OSD");
TAP_AUTHOR_NAME("TOPFIELD");
TAP_DESCRIPTION("It is sample tap program");
TAP_ETCINFO(__DATE__);

static byte _iboxShow = 0;
static word _iboxRgn;
static word _iboxTempRgn;

static byte _volShow = 0;
static word _volRgn;
static word _volTempRgn;

static dword _svcStatus[2] = {0,0};

#define IBOX_X		((720-_iboxGd.width)/2)
#define IBOX_Y		400
#define IBOX_W		(_iboxGd.width)
#define IBOX_H		(_iboxGd.height)

#define VOL_X 		IBOX_X
#define VOL_Y		(IBOX_Y - (_volGd.height) - 5)
#define VOL_Y2		(IBOX_Y + IBOX_H - (_volGd.height))
#define VOL_W		(_volGd.width)
#define VOL_H		(_volGd.height)

#define VOL_TimeOut		200

static char textBuf[1024];

void ShowVol( void );
void HideVol( void );

void ShowInfoBox( void )
{
	int svcType;
	int svcNum;
	int signalQuility, signalLevel;
	TYPE_TapChInfo chInfo;
	word mjd;
	byte hour, min, sec;
	int x, w;
	TYPE_TapEvent *presentEvent;

	if( _iboxShow == 0 )
	{
		_iboxShow = 1;
		if( _volShow )
		{
			HideVol();
			ShowVol();
		}
		_iboxRgn = TAP_Osd_Create( IBOX_X, IBOX_Y, IBOX_W, IBOX_H, 0, 0 );
		_iboxTempRgn = TAP_Osd_Create( IBOX_X, IBOX_Y, IBOX_W, IBOX_H, 0, OSD_Flag_MemRgn );
	}
	
	TAP_Osd_PutGd( _iboxTempRgn, 0 , 0, &_iboxGd, TRUE );

	TAP_Channel_GetCurrent( &svcType, &svcNum );
	TAP_Channel_GetInfo( svcType, svcNum, &chInfo );

	// service name
	TAP_Osd_PutString1926( _iboxTempRgn, 10, 10, -1, chInfo.chName, COLOR_White, COLOR_None );

	// time
	TAP_GetTime( &mjd, &hour, &min, &sec);
	if( hour <= 12 )
		sprintf( textBuf, "AM %02d:%02d:%02d", hour, min, sec );
	else
		sprintf( textBuf, "PM %02d:%02d:%02d", hour%12, min, sec );
	TAP_Osd_PutString1622( _iboxTempRgn, IBOX_W-105, 15, -1, textBuf, COLOR_Yellow, COLOR_None );

	// service numer
	TAP_Osd_FillBox( _iboxTempRgn, 8, 46, 70, 70, RGB( 8, 10, 15 ) );
	sprintf( textBuf, "%d", svcNum + 1 );
	w = TAP_Osd_GetW( textBuf, 0, FNT_Size_1926 );
	TAP_Osd_PutString1926( _iboxTempRgn, 45 - w, 70, -1, textBuf, RGB( 25, 30, 25), COLOR_None );

	// event info
	presentEvent = TAP_GetCurrentEvent( svcType, svcNum );
	if( presentEvent->evtId )
	{
		sprintf( textBuf, "%s", presentEvent->eventName );
		TAP_Osd_PutString1622( _iboxTempRgn, 85, 50, IBOX_W-10, textBuf, RGB( 25, 25, 30), COLOR_None );
		sprintf( textBuf, "%s", presentEvent->description );
		TAP_Osd_PutString1622( _iboxTempRgn, 85, 70, IBOX_W-10, textBuf, RGB( 25, 25, 30), COLOR_None );
	}
	else
	{
		sprintf( textBuf, "No Event Information", presentEvent->eventName );
		TAP_Osd_PutString1622( _iboxTempRgn, 85, 50, IBOX_W-10, textBuf, RGB( 25, 25, 30), COLOR_None );
	}

	// dolby
	x = 85;	
	if( chInfo.dolbyTrack != 0xff )
	{
		sprintf( textBuf, "Dolby Digital" );
		w = TAP_Osd_GetW( textBuf, 0, FNT_Size_1419 ) + 2;
		TAP_Osd_PutString1419( _iboxTempRgn, x, 100, x+w, textBuf, RGB( 2, 2, 10), RGB( 20, 20, 5 ) );
		x = x + w + 5;
	}

	// multifeed
	if( chInfo.multifeed )
	{
		sprintf( textBuf, "Multifeed" );
		w = TAP_Osd_GetW( textBuf, 0, FNT_Size_1419 ) + 2;
		TAP_Osd_PutString1419( _iboxTempRgn, x, 100, x+w, textBuf, RGB( 2, 2, 10), RGB( 20, 20, 5 ) );
		x = x + w + 5;
	}

	// signal level/quility
	signalLevel = TAP_GetSignalLevel();
	TAP_Osd_FillBox( _iboxTempRgn, IBOX_W - 222, 100, 204, 10, RGB( 1, 1, 3 ) );
	TAP_Osd_FillBox( _iboxTempRgn, IBOX_W - 220, 102, signalLevel * 200 / 100, 6, RGB( 25, 23, 10 ) );

	signalQuility = TAP_GetSignalQuality();
	TAP_Osd_FillBox( _iboxTempRgn, IBOX_W - 222, 110, 204, 10, RGB( 1, 1, 3 ) );
	TAP_Osd_FillBox( _iboxTempRgn, IBOX_W - 220, 112, signalQuility * 200 / 100, 6, RGB( 10, 25, 10 ) );

	// service status
	if( _svcStatus[CHANNEL_Main] & SVCSTATUS_MASK_Error )
	{
		sprintf( textBuf, "Service Error" );
		w = TAP_Osd_GetW( textBuf, 0, FNT_Size_1419 ) + 2;
		TAP_Osd_PutString1419( _iboxTempRgn, x, 100, x+w, textBuf, RGB( 30, 28, 28), RGB( 20, 5, 5 ) );
		x = x + w + 5;
	}

	// copy to screen
	TAP_Osd_Copy( _iboxTempRgn, _iboxRgn, 0, 0, IBOX_W, IBOX_H, 0, 0, FALSE );
}

void HideInfoBox( void )
{
	if( _iboxShow == 0 )
		return;
	
	_iboxShow = 0;
	TAP_Osd_Delete( _iboxRgn );
	TAP_Osd_Delete( _iboxTempRgn );
}

void ShowVol( void )
{
	int vol;
	int rgnY;

	if( _volShow == 0 )
	{
		rgnY = (_iboxShow)?VOL_Y:VOL_Y2;
		_volRgn = TAP_Osd_Create( VOL_X, rgnY, VOL_W, VOL_H, 0, 0 );
		_volTempRgn = TAP_Osd_Create(  VOL_X, rgnY, VOL_W, VOL_H, 0, OSD_Flag_MemRgn );
		_volShow = 1;
	}

	TAP_Osd_PutGd( _volTempRgn, 0 , 0, &_volGd, TRUE );

	vol = (TAP_GetSystemVar( SYSVAR_Vol ) & 0x7f);

	TAP_Osd_FillBox( _volTempRgn, 5, 15, vol * (VOL_W-10) / MAX_Vol,  VOL_H-30,  RGB( vol * 31 / MAX_Vol, 25, 5 ) );
	if( TAP_GetSystemVar( SYSVAR_Vol ) & 0x80 )	// MUTE
	{
		TAP_Osd_PutString1622( _volTempRgn, 10, 14, -1, "MUTE", COLOR_Red, COLOR_None );
	}

	// copy to screen
	TAP_Osd_Copy( _volTempRgn, _volRgn, 0, 0, VOL_W, VOL_H, 0, 0, FALSE );
}

void HideVol( void )
{
	if( _volShow == 0 )
		return;
	_volShow = 0;
	TAP_Osd_Delete( _volRgn );
	TAP_Osd_Delete( _volTempRgn );
}

dword TAP_EventHandler( word event, dword param1, dword param2 )
{
	int r, t;
	dword mainState, subState;
	static dword oldTick = 0;
	static dword volTick = 0;

	TYPE_PlayInfo playInfo;

	if( event == EVT_SVCSTATUS )
	{
		_svcStatus[(param1 & SVCSTATUS_MASK_MainSub) >> 28 ] = (param1 & SVCSTATUS_MASK_Error);
	}

	TAP_GetState( &mainState, &subState );
    
	if( mainState != STATE_Normal || subState !=  SUBSTATE_Normal )
	{
		return param1;
	}

	if( _volShow )
	{
		if( volTick + VOL_TimeOut < TAP_GetTick() )
			HideVol();
	}

	if( event == EVT_IDLE )
	{
		if( oldTick !=  TAP_GetTick() )
		{
			if( _iboxShow )
				ShowInfoBox();

			if( _volShow )
				ShowVol();
			
			oldTick = TAP_GetTick();
		}
	}

	if( event != EVT_KEY )
      	return param1;

	switch( param1 )
	{
		case  RKEY_Info :
			if( _iboxShow == 0 )
			{
				ShowInfoBox();
			}
			else
			{
				HideInfoBox();
			}
			return 0;
			
		case RKEY_Ok :
		case RKEY_Menu :
		case RKEY_Guide :
		case RKEY_PlayList :
			HideInfoBox();
			HideVol();
			return param1;

		case RKEY_Exit :
			HideInfoBox();
			HideVol();
			/*
			TAP_SysOsdControl( SYSOSD_InfoBox, TRUE );
			TAP_SysOsdControl( SYSOSD_ServiceStatus, TRUE );
			TAP_SysOsdControl( SYSOSD_VolumeBar, TRUE );
			TAP_Exit();
			*/
			return param1;

		case RKEY_VolDown :
		case RKEY_VolUp :
			ShowVol();
			volTick = TAP_GetTick();
			return param1;
	}

	return param1;
}

int TAP_Main(void)
{
	int i;

	_iboxGd.data = _iboxGd.data + _tap_startAddr;
	_volGd.data = _volGd.data + _tap_startAddr;

	TAP_EnterNormal();
	
	TAP_SysOsdControl( SYSOSD_InfoBox, FALSE );
	TAP_SysOsdControl( SYSOSD_ServiceStatus, FALSE );
	TAP_SysOsdControl( SYSOSD_VolumeBar, FALSE );
    
	return 1;
}

